@@ -41,6 +41,7 @@
 #include "theme.h"
 #include "tool.h"
 #include "tools.h"
+#include "target.h"
 #include "window.h"
 #include "zoomdlg.h"
 #include <gcu/filechooser.h>
@@ -238,7 +239,7 @@ static void on_config_changed (GConfClient *client, guint cnxn_id, GConfEntry *e
 }
 
 Application::Application ():
-	gcu::Application ("GChemPaint", DATADIR, PACKAGE, "gchempaint")
+	gcu::Application ("GChemPaint", DATADIR, "gchempaint-unstable", "gchempaint")
 {
 	m_CurZ = 6;
 	m_pActiveDoc = NULL;
@@ -573,7 +574,7 @@ void Application::SaveWithBabel (string const &filename, const gchar *mime_type,
 
 void Application::OpenWithBabel (string const &filename, const gchar *mime_type, Document* pDoc)
 {
-	char *old_num_locale;
+	string old_num_locale;
 	bool bNew = (pDoc == NULL || (!pDoc->GetEmpty () && ! pDoc->GetDirty ())), local;
 	GnomeVFSFileInfo *info = NULL;
 	bool result = false, read_only = false;
@@ -597,7 +598,7 @@ void Application::OpenWithBabel (string const &filename, const gchar *mime_type,
 			gnome_vfs_uri_unref (uri);
 			if (ifs.fail ())
 				throw (int) 1;
-			old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
+			old_num_locale = setlocale (LC_NUMERIC, NULL);
 			setlocale(LC_NUMERIC, "C");
 			OBMol Mol;
 			OBConversion Conv;
@@ -611,8 +612,7 @@ void Application::OpenWithBabel (string const &filename, const gchar *mime_type,
 				if (!result)
 					break;
 			}
-			setlocale (LC_NUMERIC, old_num_locale);
-			g_free (old_num_locale);
+			setlocale (LC_NUMERIC, old_num_locale.c_str ());
 			ifs.close ();
 		} else {
 			char *buf;
@@ -620,7 +620,7 @@ void Application::OpenWithBabel (string const &filename, const gchar *mime_type,
 			if (gnome_vfs_read_entire_file (filename.c_str (), &size, &buf) != GNOME_VFS_OK)
 				throw 1;
 			istringstream iss (buf);
-			old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
+			old_num_locale = setlocale (LC_NUMERIC, NULL);
 			setlocale(LC_NUMERIC, "C");
 			OBMol Mol;
 			OBConversion Conv;
@@ -634,13 +634,13 @@ void Application::OpenWithBabel (string const &filename, const gchar *mime_type,
 				if (!result)
 					break;
 			}
-			setlocale (LC_NUMERIC, old_num_locale);
-			g_free (old_num_locale);
+			setlocale (LC_NUMERIC, old_num_locale.c_str ());
 			g_free (buf);
 		}
 		if (!result)
 		{
-			if (bNew) delete pDoc;
+			if (bNew)
+				pDoc->GetWindow ()->Destroy ();
 			throw (int) 2;
 		}
 		pDoc->SetFileName (filename, mime_type);
@@ -765,7 +765,7 @@ void Application::OpenGcp (string const &filename, Document* pDoc)
 		g_free(old_time_locale);
 		if (!result) {
 			if (create)
-				delete pDoc;
+				pDoc->GetWindow ()->Destroy ();
 			throw (int) 4;
 		}
 		if (!(info->permissions & (GNOME_VFS_PERM_USER_WRITE | GNOME_VFS_PERM_GROUP_WRITE)))
@@ -955,15 +955,15 @@ void Application::RegisterToolbar (char const *name, int index)
 		ToolbarNames[index] = name;
 }
 
-void Application::AddWindow (Window *window)
+void Application::AddTarget (Target *target)
 {
-	m_Windows.insert (window);
+	m_Targets.insert (target);
 	NotifyIconification (false);
 }
 
-void Application::DeleteWindow (Window *window)
+void Application::DeleteTarget (Target *target)
 {
-	m_Windows.erase (window);
+	m_Targets.erase (target);
 	ShowTools (false);
 }
 
@@ -974,11 +974,11 @@ void Application::NotifyIconification (bool iconified)
 	}
 }
 
-void Application::NotifyFocus (bool has_focus, Window *window)
+void Application::NotifyFocus (bool has_focus, Target *target)
 {
-	if (window) {
-		m_pActiveWin = window;
-		m_pActiveDoc = window->GetDocument ();
+	if (target) {
+		m_pActiveTarget = target;
+		m_pActiveDoc = target->GetDocument ();
 		m_pActiveTool->Activate ();
 		if (has_focus)
 			ShowTools (true);
@@ -987,16 +987,19 @@ void Application::NotifyFocus (bool has_focus, Window *window)
 
 void Application::CloseAll ()
 {
-	while (!m_Windows.empty ())
-		if (!(*m_Windows.begin ())->Close ())
+	while (!m_Targets.empty ())
+		if (!(*m_Targets.begin ())->Close ())
 			return;
 }
 
 void Application::ActivateWindowsActionWidget (const char *path, bool activate)
 {
-	std::set<Window*>::iterator i, iend = m_Windows.end ();
-	for (i = m_Windows.begin (); i != iend; i++)
-		(*i)->ActivateActionWidget (path, activate);
+	std::set<Target*>::iterator i, iend = m_Targets.end ();
+	for (i = m_Targets.begin (); i != iend; i++) {
+		Window *window = dynamic_cast<Window*> (*i);
+		if (window)
+			window->ActivateActionWidget (path, activate);
+	}
 }
 
 void Application::OnConfigChanged (GConfClient *client, guint cnxn_id, GConfEntry *entry)
@@ -1053,4 +1056,24 @@ void Application::BuildMenu (GtkUIManager *manager)
 		(*i) (manager);
 }
 
+struct option_data {
+	GOptionEntry const *entries;
+	char const *translation_domain;
+};
+
+void Application::RegisterOptions (GOptionEntry const *entries, char const *translation_domain)
+{
+	struct option_data d;
+	d.entries = entries;
+	d.translation_domain = translation_domain;
+	m_Options.push_back (d);
+}
+
+void Application::AddOptions (GOptionContext *context)
+{
+	list<option_data>::iterator i, end = m_Options.end ();
+	for (i = m_Options.begin (); i != end; i++)
+		g_option_context_add_main_entries (context, (*i).entries, (*i).translation_domain);
+}
+
 }	//	namespace gcp
