@@ -55,7 +55,7 @@ static void do_build_inchi (Molecule* pMol)
 
 static void do_build_smiles (Molecule* pMol)
 {
-	pMol->BuildSmiles ();
+	pMol->BuildSMILES ();
 }
 
 static void do_show_webbook (Molecule* pMol)
@@ -77,6 +77,7 @@ Molecule::Molecule (TypeId Type): Object (Type)
 {
 	m_Alignment = NULL;
 	m_Changed = true;
+	m_IsResidue = false;
 }
 
 Molecule::Molecule (Atom* pAtom): Object (MoleculeType)
@@ -86,6 +87,7 @@ Molecule::Molecule (Atom* pAtom): Object (MoleculeType)
 	delete pChain;
 	m_Alignment = NULL;
 	m_Changed = true;
+	m_IsResidue = false;
 }
 
 Molecule::~Molecule ()
@@ -108,6 +110,8 @@ void Molecule::AddAtom (Atom* pAtom)
 	m_Atoms.remove (pAtom); // avoid duplicates
 	m_Atoms.push_back (pAtom);
 	AddChild (pAtom);
+	if (!pAtom->GetZ ())
+		m_IsResidue = true;
 }
 
 void Molecule::AddFragment (Fragment* pFragment)
@@ -362,6 +366,21 @@ bool Molecule::Load (xmlNodePtr node)
 		AddAtom ((Atom*) pObject);
 		child = GetNextNodeByName (child->next, "atom");
 	}
+	// FIXME, the following looks like a kludge
+	child = GetNodeByName (node, "pseudo-atom");
+	while (child) {
+		pObject = CreateObject ("pseudo-atom", pDoc);
+		if (pDoc)
+			AddChild (pObject);
+		if (!pObject->Load (child)) {
+			delete pObject;
+			return false;
+		}
+		if (pDoc)
+			pDoc->AddAtom ((Atom*) pObject);
+		AddAtom ((Atom*) pObject);
+		child = GetNextNodeByName (child->next, "pseudo-atom");
+	}
 	
 	child = GetNodeByName (node, "fragment");
 	while (child) {
@@ -495,13 +514,14 @@ double Molecule::GetYAlign ()
 
 bool Molecule::BuildContextualMenu (GtkUIManager *UIManager, Object *object, double x, double y)
 {
+	if (m_IsResidue)
+		return false;
 	bool result = false;
 	GtkActionGroup *group = gtk_action_group_new ("molecule");
 	GtkAction *action;
 	action = gtk_action_new ("Molecule", _("Molecule"), NULL, NULL);
 	gtk_action_group_add_action (group, action);
 	g_object_unref (action);
-	g_object_unref (action);
 	if (!m_Fragments.size ()) {
 		if (((Document*) GetDocument ())->GetApplication ()->HaveGhemical ()) {
 			action = gtk_action_new ("ghemical", _("Export molecule to Ghemical"), NULL, NULL);
@@ -527,7 +547,7 @@ bool Molecule::BuildContextualMenu (GtkUIManager *UIManager, Object *object, dou
 			g_object_unref (action);
 			gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='pubchem'/></menu></popup></ui>", -1, NULL);
 		}
-		action = gtk_action_new ("smiles", _("Generate Smiles"), NULL, NULL);
+		action = gtk_action_new ("smiles", _("Generate SMILES"), NULL, NULL);
 		g_signal_connect_swapped (action, "activate", G_CALLBACK (do_build_smiles), this);
 		gtk_action_group_add_action (group, action);
 		g_object_unref (action);
@@ -761,7 +781,7 @@ void Molecule::BuildInChI ()
 	m_Changed = false;
 }
 
-void Molecule::BuildSmiles ()
+void Molecule::BuildSMILES ()
 {
 	OBMol Mol;
 	OBConversion Conv;
@@ -848,4 +868,34 @@ char const *Molecule::GetInChI ()
 	return m_InChI.c_str ();
 }
 
+std::string Molecule::GetRawFormula ()
+{
+	ostringstream ofs;
+
+	if (!m_Fragments.size ()) {
+		// we do not support fragmentsat the moment
+		map<string, int> elts;
+		list<Atom*>::iterator ia, enda = m_Atoms.end ();
+		for (ia = m_Atoms.begin(); ia != enda; ia++) {
+			if ((*ia)->GetZ () == 0)
+				continue;
+			elts[(*ia)->GetSymbol ()]++;
+			elts["H"] += (*ia)->GetAttachedHydrogens ();
+		}
+		if (elts["C"] > 0) {
+			ofs << "C" << elts["C"];
+			elts.erase ("C");
+		}
+		if (elts["H"] > 0) {
+			ofs << "H" << elts["H"];
+			elts.erase ("H");
+		}
+		map<string, int>::iterator is, isend = elts.end ();
+		for (is = elts.begin (); is != isend; is++)
+			ofs << (*is).first << (*is).second;
+	}
+
+	return ofs.str ();
+}
+
 }	//	namespace gcp
