@@ -2,7 +2,7 @@
 
 /* 
  * Gnome Chemistry Utils
- * programs/gchemcalc.cc 
+ * programs/calc/gchemcalc.cc 
  *
  * Copyright (C) 2005-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
@@ -23,9 +23,13 @@
  */
 
 #include "config.h"
+// FIXME: "the following lines should be removed for stable releases"
+#undef PACKAGE
+#define PACKAGE "gchemutils-unstable" 
 #include <gcu/application.h>
 #include <gcu/element.h>
 #include <gcu/formula.h>
+#include <gcu/residue.h>
 #include <glib/gi18n.h>
 #include <glade/glade.h>
 #include <gtk/gtkmain.h>
@@ -59,9 +63,9 @@
 #include <gsf/gsf-input-memory.h>
 #include <gsf/gsf-output-memory.h>
 #include <libgnomevfs/gnome-vfs.h>
-#include <iostream>
+#include <libxml/tree.h>
 #include <cmath>
-#include <cstring>
+#include <iostream>
 
 using namespace gcu;
 
@@ -79,11 +83,42 @@ public:
 	GogPlot *plot;
 	GogSeries *series;
 	GtkListStore *pclist;
+
+private:
+	void ParseNodes (xmlNodePtr node);
 };
 
-GChemCalc::GChemCalc (): Application ("gchemcalc"),
+GChemCalc::GChemCalc (): Application ("gchemcalc-unstable"),
 formula ("")
 {
+	// Load residues
+	xmlDocPtr doc;
+	char *name;
+	doc = xmlParseFile (PKGDATADIR"/residues.xml");
+	if (doc) {
+		if (!strcmp ((char*) doc->children->name, "residues"))
+			ParseNodes (doc->children->children);
+		xmlFreeDoc (doc);
+	}
+	name = g_strconcat (getenv ("HOME"), "/.gchemutils/residues.xml", NULL);
+	if (g_file_test (name, G_FILE_TEST_EXISTS) && (doc = xmlParseFile (name))) {
+		if (!strcmp ((char*) doc->children->name, "residues"))
+			ParseNodes (doc->children->children);
+		xmlFreeDoc (doc);
+	}
+	g_free (name);
+}
+
+void GChemCalc::ParseNodes (xmlNodePtr node)
+{
+	Residue* r;
+	while (node) {
+		if (!strcmp ((char*) node->name, "residue")) {
+			r = new Residue ();
+			r->Load (node);
+		}
+		node = node->next;
+	}
 }
 
 GChemCalc *App;
@@ -178,7 +213,7 @@ static void cb_entry_active (GtkEntry *entry, gpointer data)
 		} else {
 			if (prec < 0) {
 				// round the value to replace not significant figures by 0s.
-				double offs = pow (10.0, (double) prec);
+				double offs = pow10 (prec);
 				weight = rint (weight * offs) / offs;
 			}
 			format = artificial? g_strdup ("(%.0f)"): g_strdup ("%.0f");
@@ -499,8 +534,9 @@ int main (int argc, char *argv[])
 		argv ++;
 	}
 
+// FIXME "the following line should be edited for stable releases"
 	if (argc > 1) {
-		cout << _("For usage see: gchemcalc [-?|--help]") << endl;
+		cout << _("For usage see: gchemcalc-unstable [-?|--help]") << endl;
 		return -1;
 	}
 	
@@ -601,7 +637,7 @@ int main (int argc, char *argv[])
 	g_signal_connect (GTK_OBJECT (w), "activate",
 		 G_CALLBACK (cb_entry_active),
 		 window);
-	gcu_element_load_databases ((char*) "isotopes", NULL);
+	gcu_element_load_databases ("isotopes", NULL);
 	if (argc == 1){
 		gtk_entry_set_text (GTK_ENTRY (w), argv[0]);
 		cb_entry_active (GTK_ENTRY (w), window);
